@@ -3,45 +3,40 @@
  * The Auth_imap:: class provides an IMAP implementation of the Horde
  * authentication system.
  *
- * Optional parameters:
- * ====================
- *  'folder'         --  The initial folder / mailbox to open.
- *                       Should be null for 'imap' and 'nntp' protocols.
- *                       DEFAULT: null
- *  'hostspec'       --  The hostname or IP address of the server.
- *                       DEFAULT: 'localhost'
- *  'port'           --  The server port to which we will connect.
- *                       IMAP is generally 143, while IMAP-SSL is generally 993.
- *                       DEFAULT: 143
- *  'protocol'       --  The connection protocol (e.g. 'imap', 'pop3', 'nntp').
- *                       Protocol is one of 'imap/notls' (or only 'imap' if you
- *                       have a c-client version 2000c or older), 'imap/ssl',
- *                       or 'imap/ssl/novalidate-cert' (for a self-signed
- *                       certificate).
- *                       DEFAULT: 'imap'
- *  'admin_user'     --  The name of a user with admin privileges.
- *                       DEFAULT: null
- *  'admin_password' --  The password of the adminstrator.
- *                       DEFAULT: null
- *  'userhierarchy'  --  The hierarchy where user mailboxes are stored.
- *                       DEFAULT: 'user.'
- *  'dsn'            --  The full IMAP connection string.
- *                       If not present, this is built from 'hostspec', 'port'
- *                       and 'protocol' parameters.
+ * Optional parameters:<pre>
+ *  'hostspec'        The hostname or IP address of the server.
+ *                    DEFAULT: 'localhost'
+ *  'port'            The server port to which we will connect.
+ *                    IMAP is generally 143, while IMAP-SSL is generally 993.
+ *                    DEFAULT: 143
+ *  'protocol'        The connection protocol (e.g. 'imap', 'pop3', 'nntp').
+ *                    Protocol is one of 'imap/notls' (or only 'imap' if you
+ *                    have a c-client version 2000c or older), 'imap/ssl',
+ *                    or 'imap/ssl/novalidate-cert' (for a self-signed
+ *                    certificate).
+ *                    DEFAULT: 'imap'
+ *  'admin_user'      The name of a user with admin privileges.
+ *                    DEFAULT: null
+ *  'admin_password'  The password of the adminstrator.
+ *                    DEFAULT: null
+ *  'userhierarchy'   The hierarchy where user mailboxes are stored.
+ *                    DEFAULT: 'user.'
+ *  'dsn'             The full IMAP connection string.
+ *                    If not present, this is built from 'hostspec', 'port'
+ *                    and 'protocol' parameters.</pre>
  *
  *
- * If setting up as Horde auth handler in conf.php, this is a sample entry:
- *   $conf['auth']['params']['folder'] = 'INBOX';
+ * If setting up as Horde auth handler in conf.php, this is a sample entry:<pre>
  *   $conf['auth']['params']['hostspec'] = 'imap.example.com';
  *   $conf['auth']['params']['port'] = 143;
- *   $conf['auth']['params']['protocol'] = 'imap/notls/novalidate-cert';
+ *   $conf['auth']['params']['protocol'] = 'imap/notls/novalidate-cert';</pre>
  *
- * 
- * $Horde: framework/Auth/Auth/imap.php,v 1.28.10.1 2005/01/03 12:18:52 jan Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Gaudenz Steinlin <gaudenz.steinlin@id.unibe.ch>
- * Copyright 2004-2005 Jan Schneider <jan@horde.org>
+ * $Horde: framework/Auth/Auth/imap.php,v 1.28.10.10 2007/01/02 13:54:07 jan Exp $
+ *
+ * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Gaudenz Steinlin <gaudenz.steinlin@id.unibe.ch>
+ * Copyright 2004-2007 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -49,7 +44,6 @@
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Gaudenz Steinlin <gaudenz.steinlin@id.unibe.ch>
  * @author  Jan Schneider <jan@horde.org>
- * @version $Revision: 1.28.10.1 $
  * @since   Horde 1.3
  * @package Horde_Auth
  */
@@ -58,18 +52,15 @@ class Auth_imap extends Auth {
     /**
      * Constructs a new IMAP authentication object.
      *
-     * @access public
-     *
-     * @param optional array $params  A hash containing connection parameters.
+     * @param array $params  A hash containing connection parameters.
      */
     function Auth_imap($params = array())
     {
         if (!Util::extensionExists('imap')) {
-            Horde::fatal(PEAR::raiseError(_("Auth_imap: Required IMAP extension not found."), __FILE__, __LINE__));
+            Horde::fatal(_("Auth_imap: Required IMAP extension not found."), __FILE__, __LINE__);
         }
 
         $default_params = array(
-            'folder' => null,
             'hostspec' => 'localhost',
             'port' => '143',
             'protocol' => 'imap',
@@ -85,13 +76,10 @@ class Auth_imap extends Auth {
 
         /* Create DSN string. */
         if (!isset($this->_params['dsn'])) {
-            $this->_params['dsn'] = sprintf('{%s:%d/%s}', 
+            $this->_params['dsn'] = sprintf('{%s:%d/%s}',
                                             $this->_params['hostspec'],
                                             $this->_params['port'],
                                             $this->_params['protocol']);
-            if (!empty($this->_params['folder'])) {
-                $this->_params['dsn'] .= $this->_params['folder'];
-            }
         }
     }
 
@@ -109,7 +97,7 @@ class Auth_imap extends Auth {
     function _authenticate($userId, $credentials)
     {
         if (empty($credentials['password'])) {
-            Horde::fatal(PEAR::raiseError(_("No password provided for IMAP authentication.")), __FILE__, __LINE__);
+            Horde::fatal(_("No password provided for IMAP authentication."), __FILE__, __LINE__);
         }
 
         $imap = @imap_open($this->_params['dsn'], $userId,
