@@ -24,7 +24,7 @@
  * The table structure for the DataTree system is in
  * scripts/sql/horde_datatree.sql.
  *
- * $Horde: framework/DataTree/DataTree/sql.php,v 1.156.2.33 2007/07/14 13:06:45 jan Exp $
+ * $Horde: framework/DataTree/DataTree/sql.php,v 1.156.2.34 2007/12/06 18:10:39 jan Exp $
  *
  * Copyright 1999-2007 Stephane Huther <shuther@bigfoot.com>
  * Copyright 2001-2007 Chuck Hagenbuch <chuck@horde.org>
@@ -504,7 +504,7 @@ class DataTree_sql extends DataTree {
         }
 
         if (parent::exists($fullname)) {
-            return PEAR::raiseError(_("Already exists"));
+            return PEAR::raiseError(sprintf(_("\"%s\" already exists"), $fullname));
         }
 
         $query = 'INSERT INTO ' . $this->_params['table'] .
