@@ -2,20 +2,19 @@
 /**
  * The IMAP_Search:: class performs complex searching of an IMAP mailbox.
  *
- * Classes to help with complex searching of an IMAP mailbox.
- * The built-in PHP search() function only allows IMAPv2 search queries
- * (see RFC 1176).  This library allows more complex searches to be
- * created (e.g. OR searches, searching specific headers).
+ * Classes to help with complex searching of an IMAP mailbox.  The built-in
+ * PHP search() function only allows IMAPv2 search queries (see RFC 1176).
+ * This library allows more complex searches to be created (e.g. OR searches,
+ * searching specific headers).
  *
- * $Horde: framework/IMAP/IMAP/Search.php,v 1.29.10.1 2005/01/03 12:19:02 jan Exp $
+ * $Horde: framework/IMAP/IMAP/Search.php,v 1.29.10.27 2007/04/25 18:26:19 jan Exp $
  *
- * Copyright 2003-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2003-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
- * See the enclosed file COPYING for license information (GPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
+ * See the enclosed file COPYING for license information (GPL). If you did not
+ * receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.29.10.1 $
  * @since   Horde 3.0
  * @package Horde_IMAP
  */
@@ -24,43 +23,48 @@ class IMAP_Search {
     /**
      * The headers cache.
      *
-     * @var array $_headers
+     * @var array
      */
     var $_headers = array();
 
     /**
      * The results cache.
      *
-     * @var array $_result
+     * @var array
      */
     var $_result = array();
 
     /**
-     * Are we dealing with a POP3 connection?
+     * The paramater list.
      *
-     * @var boolean $_pop3
+     * @var array
      */
-    var $_pop3 = false;
+    var $_params = array();
+
+    /**
+     * The charset of the search values.
+     *
+     * @var string
+     */
+    var $_charset;
 
     /**
      * Internal flag used by searchMailbox().
      *
-     * @var integer $_searchflag
+     * @var integer
      */
     var $_searchflag = 0;
 
     /**
-     * Returns a reference to the global IMAP_Search object,
-     * only creating it if it doesn't already exist.
+     * Returns a reference to the global IMAP_Search object, only creating it
+     * if it doesn't already exist.
      *
-     * This method must be invoked as:
-     *   $imap_search = &IMAP_Search::singleton();
+     * This method must be invoked as:<code>
+     *   $imap_search = &IMAP_Search::singleton();</code>
      *
-     * @access public
+     * @param array $params  Any parameters the constructor may need.
      *
-     * @param optional array $params  Any parameters the constructor may need.
-     *
-     * @return object IMAP_Search  The IMAP_Search instance.
+     * @return IMAP_Search  The IMAP_Search instance.
      */
     function &singleton($params = array())
     {
@@ -74,38 +78,35 @@ class IMAP_Search {
     }
 
     /**
-     * Constructor
+     * Constructor.
      *
-     * @access public
-     *
-     * @param optional array $params  A hash containing the following entries:
-     *                                'pop3' => boolean (using POP3
-     *                                                   connection?)
+     * @param array $params  A hash containing the following entries:<pre>
+     *                       'pop3' => boolean (using POP3 connection?)
+     *                       'charset' => string (charset of search values)
+     *                       'no_imap_search' => true if the IMAP server does
+     *                                           not support the charset in
+     *                                           the search command
+     *                       </pre>
      */
     function IMAP_Search($params = array())
     {
-        if (isset($params['pop3'])) {
-            $this->_pop3 = $params['pop3'];
-        }
+        $this->_params = $params;
     }
 
     /**
-     * Searches messages by ALL headers (rather than the limited set
-     * provided by imap_search()).
+     * Searches messages by ALL headers (rather than the limited set provided
+     * by imap_search()).
      *
-     * @access public
-     *
-     * @param object IMAP_Search_Query $query  The search query.
-     * @param resource &$imap                  An IMAP resource stream.
-     * @param string $mbox                     The name of the mailbox to
-     *                                         search. For POP3, this should
-     *                                         be empty.
+     * @param IMAP_Search_Query $query  The search query.
+     * @param resource $imap            An IMAP resource stream.
+     * @param string $mbox              The name of the mailbox to search. For
+     *                                  POP3, this should be empty.
      *
      * @return array  The list of indices that match the search rules in the
      *                current mailbox.
      *                Returns PEAR_Error on error.
      */
-    function searchMailbox($query, &$imap, $mbox)
+    function searchMailbox($query, $imap, $mbox)
     {
         /* Check for IMAP extension. */
         if (!Util::extensionExists('imap')) {
@@ -113,8 +114,10 @@ class IMAP_Search {
         }
 
         /* Open to the correct mailbox. */
-        if (!$this->_pop3) {
-            @imap_reopen($imap, $mbox);
+        if (!$this->_params['pop3']) {
+            if (!@imap_reopen($imap, $mbox, OP_READONLY)) {
+                return array();
+            }
         }
 
         /* Clear the search flag. */
@@ -124,54 +127,73 @@ class IMAP_Search {
     }
 
     /**
-     * Search the mailbox and sort the results.
+     * Searches the mailbox and sorts the results.
      *
-     * @access public
-     *
-     * @param object IMAP_Search_Query $query  The search query.
-     * @param resource &$imap                  An IMAP resource stream.
-     * @param string $mbox                     The name of the mailbox to
-     *                                         search.
-     * @param integer $criteria                The criteria to sort by
-     *                                         (see imap_sort()).
-     * @param optional integer $dir            1 for reverse sorting.
+     * @param IMAP_Search_Query $query  The search query.
+     * @param resource $imap            An IMAP resource stream.
+     * @param string $mbox              The name of the mailbox to search.
+     * @param integer $sortby           The criteria to sort by (see
+     *                                  imap_sort()).
+     * @param integer $sortdir          1 for reverse sorting.
      *
      * @return array  The list of indices that match the search rules in the
      *                current mailbox and sorted.
      *                Returns PEAR_Error on error.
      */
-    function searchSortMailbox($query, &$imap, $mbox, $criteria, $dir = 0)
+    function searchSortMailbox($query, $imap, $mbox, $sortby, $sortdir = 0)
     {
         $indices = $this->searchMailbox($query, $imap, $mbox);
         if (is_a($indices, 'PEAR_Error')) {
             return $indices;
         } else {
-            $indices_sort = @imap_sort($imap, $criteria, $dir, SE_UID);
+            $indices_sort = @imap_sort($imap, $sortby, $sortdir, SE_UID);
             return array_values(array_intersect($indices_sort, $indices));
         }
     }
 
+    /**
+     * Internal wrapper method to imap_search that allows charset aware
+     * searches if available.
+     *
+     * @access private
+     */
+    function _imapSearch($imap, $criteria)
+    {
+        if (!empty($this->_params['charset'])) {
+            if (empty($this->_params['no_imap_charset']) &&
+                version_compare(phpversion(), '4.3.3', '>=')) {
+                return @imap_search($imap, $criteria, SE_UID, $this->_params['charset']);
+            } else {
+                $criteria = String::convertCharset($criteria, $this->_params['charset'], 'US-ASCII');
+            }
+        }
+
+        return @imap_search($imap, $criteria, SE_UID);
+    }
+
     /**
      * Internal function to search the mailbox.
      *
      * @access private
      */
-    function _searchMailbox($query, &$imap, $mbox)
+    function _searchMailbox($query, $imap, $mbox)
     {
         $indices = array();
 
+        $this->_searchflag = 0;
+
         /* Do the simple searches that imap_search() can handle. */
         if (($ob = $query->build())) {
             if ($ob->not) {
-                if (!($indices1 = @imap_search($imap, $ob->flags, SE_UID))) {
+                if (!($indices1 = $this->_imapSearch($imap, $ob->flags))) {
                     $indices1 = array();
                 }
-                if (!($indices2 = @imap_search($imap, $ob->query, SE_UID))) {
+                if (!($indices2 = $this->_imapSearch($imap, $ob->query))) {
                     $indices2 = array();
                 }
                 $indices = array_diff($indices1, $indices2);
             } else {
-                if (!($indices = @imap_search($imap, $ob->fullquery, SE_UID))) {
+                if (!($indices = $this->_imapSearch($imap, $ob->fullquery))) {
                     $indices = array();
                 }
             }
@@ -204,14 +226,19 @@ class IMAP_Search {
                     $result[] = $val->uid;
                 }
             }
+            if ($this->_searchflag == 0) {
+                $this->_searchflag = 1;
+            }
             $indices = array_values(array_intersect($indices, $result));
         }
 
         /* Process AND searches now. */
-        $indices = $this->_doAndOrSearch($query->andSearch(), $imap, $mbox, 'and', $indices);
+        $indices = $this->_doAndOrSearch($query->andSearch(), $imap, $mbox,
+                                         'and', $indices);
 
         /* Process OR searches now. */
-        $indices = $this->_doAndOrSearch($query->orSearch(), $imap, $mbox, 'or', $indices);
+        $indices = $this->_doAndOrSearch($query->orSearch(), $imap, $mbox,
+                                         'or', $indices);
 
         return $indices;
     }
@@ -221,7 +248,7 @@ class IMAP_Search {
      *
      * @access private
      */
-    function _doAndOrSearch($query, &$imap, $mbox, $mode, $indices,
+    function _doAndOrSearch($query, $imap, $mbox, $mode, $indices,
                             $base = false)
     {
         if (empty($query)) {
@@ -230,6 +257,7 @@ class IMAP_Search {
 
         foreach ($query as $val) {
             if (is_a($val, 'IMAP_Search_Query')) {
+                $prevsearch = (empty($indices) && ($this->_searchflag == 0));
                 $result = $this->_searchMailbox($val, $imap, $mbox);
                 if ($mode == 'and') {
                     /* If the result is empty in an AND search, we know that
@@ -239,13 +267,13 @@ class IMAP_Search {
                         return array();
                     }
 
-                    /* If we have reached this point, and have not performed
-                       a search yet, we must use the results as the indices
+                    /* If we have reached this point, and have not performed a
+                       search yet, we must use the results as the indices
                        list. Without this check, the indices list will always
                        be empty if an AND search is the first search. */
-                    if (empty($indices) && ($this->_searchflag == 1)) {
+                    if ($prevsearch) {
                         $indices = $result;
-                        $this->_searchflag = 2;
+                        $this->_searchflag = 1;
                     } else {
                         $indices = array_values(array_intersect($indices, $result));
                     }
@@ -261,18 +289,18 @@ class IMAP_Search {
     }
 
     /**
-     * Use a PHP based functions to perform the search.
+     * Uses PHP based functions to perform the search.
      *
      * @access private
      *
-     * @param array $imap_query  The search query.
-     * @param string $flags      Any additional flags.
-     * @param resource &$imap    An IMAP resource stream.
-     * @param string $mbox       The name of the search mailbox.
+     * @param stdClass $imap_query  Search query.
+     * @param string $flags         Any additional flags.
+     * @param resource $imap        An IMAP resource stream.
+     * @param string $mbox          The name of the search mailbox.
      *
      * @return array  The list of indices that match.
      */
-    function _searchPHP($imap_query, $flags, &$imap, $mbox)
+    function _searchPHP($imap_query, $flags, $imap, $mbox)
     {
         $indices = array();
 
@@ -280,15 +308,15 @@ class IMAP_Search {
             $flags = 'ALL';
         }
 
-        if ($this->_pop3) {
+        if (!empty($this->_params['pop3'])) {
             $mbox = 'POP3';
         }
 
         $cache_key = $mbox . '|' . $flags;
 
-        /* We have to use parseMIMEHeaders() to get each header and see if
-           any field matches. Use imap_search() to get the list of message
-           indices or return empty list if no search results. */
+        /* We have to use parseMIMEHeaders() to get each header and see if any
+         * field matches. Use imap_search() to get the list of message indices
+         * or return empty list if no search results. */
         if (!isset($this->_result[$cache_key])) {
             $this->_result[$cache_key] = @imap_search($imap, $flags, SE_UID);
         }
@@ -305,8 +333,8 @@ class IMAP_Search {
         }
 
         /* Get the header/query to search for. */
-        $query = reset($imap_query);
-        $key = strtolower(key($imap_query));
+        $query = reset($imap_query->query);
+        $key = strtolower(key($imap_query->query));
 
         foreach ($this->_result[$cache_key] as $index) {
             if (!isset($this->_headers[$mbox][$index])) {
@@ -314,12 +342,22 @@ class IMAP_Search {
             }
             $h = &$this->_headers[$mbox][$index];
 
-            /* We need to do a case insensitive search on text because,
-               for example, e-mail addresses may not be caught correctly
-               (filtering for example@example.com will not catch
-                exAmple@example.com). */
-            if (isset($h[$key]) &&
-                (stristr($h[$key], strval($query)) !== false)) {
+            /* We need to do a case insensitive search on text because, for
+             * example, e-mail addresses may not be caught correctly
+             * (filtering for example@example.com will not catch
+             * exAmple@example.com). */
+            $match_success = false;
+            if (isset($h[$key])) {
+                $hdr_array = (is_array($h[$key])) ? $h[$key] : array($h[$key]);
+                foreach ($hdr_array as $val) {
+                    if (stristr($val, strval($query)) !== false) {
+                        $match_success = true;
+                        break;
+                    }
+                }
+            }
+            if (($imap_query->not && !$match_success) ||
+                (!$imap_query->not && $match_success)) {
                 $indices[] = $index;
             }
         }
@@ -330,11 +368,10 @@ class IMAP_Search {
 }
 
 /**
- * The IMAP_Search_Object:: class is used to formulate queries to be used
- * with the IMAP_Search:: class.
+ * The IMAP_Search_Object:: class is used to formulate queries to be used with
+ * the IMAP_Search:: class.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.29.10.1 $
  * @since   Horde 3.0
  * @package Horde_IMAP
  */
@@ -354,15 +391,22 @@ class IMAP_Search_Query {
     }
 
     /**
-     * Return any extended searches.
+     * Returns any extended searches.
      */
     function extendedSearch()
     {
-        return $this->_extendedSearch;
+        if (!empty($this->_extendedSearch)) {
+            $search = &new stdClass;
+            $search->not = $this->_not;
+            $search->query = $this->_extendedSearch;
+            return $search;
+        } else {
+            return null;
+        }
     }
 
     /**
-     * Return the parameters for a size search.
+     * Returns the parameters for a size search.
      */
     function sizeSearch()
     {
@@ -370,14 +414,14 @@ class IMAP_Search_Query {
             return null;
         }
         $ob = &new stdClass;
-        $ob->size = $this->_size;
+        $ob->size = (float)$this->_size;
         $ob->sizeop = $this->_sizeop;
 
         return $ob;
     }
 
     /**
-     * Return any AND searches.
+     * Returns any AND searches.
      */
     function andSearch()
     {
@@ -385,7 +429,7 @@ class IMAP_Search_Query {
     }
 
     /**
-     * Return any OR searches.
+     * Returns any OR searches.
      */
     function orSearch()
     {
@@ -393,7 +437,7 @@ class IMAP_Search_Query {
     }
 
     /**
-     * Return the flags.
+     * Returns the flags.
      */
     function flags()
     {
@@ -401,7 +445,7 @@ class IMAP_Search_Query {
     }
 
     /**
-     * Build the IMAP search query.
+     * Builds the IMAP search query.
      */
     function build()
     {
@@ -411,7 +455,7 @@ class IMAP_Search_Query {
         $search->flags = $this->flags();
 
         if (empty($this->_query)) {
-            if (empty($search->flags)) {
+            if (!empty($this->_or)) {
                 return '';
             }
             $search->query = 'ALL';
@@ -422,7 +466,7 @@ class IMAP_Search_Query {
             }
             $search->query = $this->_query;
         }
-        $search->fullquery = $search->flags . ' ' . $search->query;
+        $search->fullquery = (!empty($search->flags) ? $search->flags . ' ' : '') . $search->query;
 
         return $search;
     }
@@ -449,7 +493,8 @@ class IMAP_Search_Query {
     /* IMAP Search Flags. */
     /* There is no need to support the KEYWORD/UNKEYWORD query since the
        individual keywords have identical functionality. */
-    function _imapFlags($flag, $cmd) {
+    function _imapFlags($flag, $cmd)
+    {
         $cmd = ($flag) ? $cmd : 'UN' . $cmd;
         $this->_flags[] = $cmd;
     }
@@ -485,12 +530,13 @@ class IMAP_Search_Query {
     }
 
     /* IMAP Header Search. */
-    function header($header, $query, $not = false) {
+    function header($header, $query, $not = false)
+    {
         $header = ucfirst(rtrim($header, ':'));
         $stdHdrs = array('To', 'Cc', 'From', 'Subject');
         if ($query != '') {
             if (in_array($header, $stdHdrs)) {
-                $this->_query = String::upper($header) . ' "' . addslashes($query) . '"';
+                $this->_query = String::upper($header) . ' "' . str_replace('"', '\"', $query) . '"';
             } else {
                 $this->_extendedSearch[$header] = $query;
             }
